#!/usr/bin/env python
# -*- coding: UTF8 -*-

from xml.sax.handler import ContentHandler

class BusStopHandler(ContentHandler):
    """handler to deal with bus stops"""
    stops = []
    def __init__(self):
        self.inNode = 0
        self.isStop = 0
        self.id = 0
        self.lat = 0
        self.lon = 0
        self.name = ""
        ContentHandler.__init__(self)
    
    def startElement(self, name, attrs):
        if name == "node":
            self.inNode = 1
            self.id = attrs.get("id","")
            self.lat = attrs.get("lat","")
            self.lon = attrs.get("lon","")
        elif self.inNode:
            if name == "tag":
                type_ = attrs.get("k","")
                data = attrs.get("v","")
                if type_ == "highway" and data == "bus_stop":
                    self.isStop = 1
                if type_ == "name":
                    self.name = data
                    
    def endElement(self,name):
        if name == "node":
            if self.isStop == 1:
                #print self.id, self.name
                BusStopHandler.stops.append((self.id, self.name, self.lat, self.lon))
            self.__init__()

class TramStopHandler(ContentHandler):
    """handler to deal with trams stops"""
    stops = []
    
    def __init__(self):
        self.inNode = 0
        self.isStop = 0
        self.id = 0
        self.lat = 0
        self.lon = 0
        self.name = ""
        ContentHandler.__init__(self)
    
    def startElement(self, name, attrs):
        if name == "node":
            self.inNode = 1
            self.id = attrs.get("id","")
            self.lat = attrs.get("lat","")
            self.lon = attrs.get("lon","")
        elif self.inNode:
            if name == "tag":
                type_ = attrs.get("k","")
                data = attrs.get("v","")
                if type_ == "railway" and data == "tram_stop":
                    self.isStop = 1
                if type_ == "name":
                    self.name = data
    
    def endElement(self,name):
        if name == "node":
            if self.isStop == 1:
                #print self.id, self.name
                TramStopHandler.stops.append((self.id, self.name, self.lat, self.lon))
            self.__init__()

class RelationHandler(ContentHandler):
    """Handler to deal with relations"""
    busRelations = []
    tramRelations = []
    def __init__(self):
        ContentHandler.__init__(self)
        self.inRelation = 0
        self.isBus = 0
        self.isTram = 0
        self.id = 0
        self.ways = []
        self.nodes = []
        self.nr = 0
    
    def startElement(self, name, attrs):
        if name == "relation":
            self.inRelation = 1
            self.id = attrs.get("id","")
        elif self.inRelation:
            if name == "member":
                type_ = attrs.get("type","")
                ref = attrs.get("ref","")
                if type_ == "way":
                    self.ways.append(ref)
                if type_ == "node":
                    self.nodes.append(ref)
            if name == "tag":
                type_ = attrs.get("k","")
                data = attrs.get("v","")
                if type_ == "ref":
                    self.nr = data
                if type_ == "route" and data == "tram":
                    self.isTram = 1
                if type_ == "route" and data == "bus":
                    self.isBus = 1
                    
    def endElement(self,name):
        if name == "relation":
            if self.isBus == 1:
#                print "Bus " + self.id, self.nr
#                print "nodes:"
#                print self.nodes
#                print "ways:"
#                print self.ways
                RelationHandler.busRelations.append((self.id, self.nr, self.nodes, self.ways))
            if self.isTram == 1:
#                print "Tram " + self.id, self.nr
#                print "nodes:"
#                print self.nodes
#                print "ways:"
#                print self.ways
                RelationHandler.tramRelations.append((self.id, self.nr, self.nodes, self.ways))
            self.__init__()
            
class WayHandler(ContentHandler):
    """Halndler to deal with ways"""
    ways = []
    def __init__(self):
        ContentHandler.__init__(self)
        self.inWay = 0
        self.id = 0
        self.nodes = []
    
    def startElement(self, name, attrs):
        if name == "way":
            self.inWay = 1
            self.id = attrs.get("id","")
        elif self.inWay:
            if name == "nd":
                self.nodes.append(attrs.get("ref",""))
    
    def endElement(self,name):
        if name == "way":
#            print "way: " + self.id
#            print self.nodes
            WayHandler.ways.append((self.id, self.nodes))
            self.__init__()