#!/usr/bin/env python
# -*- coding: utf-8 -*-
# test.py

from xml.sax import make_parser
from handlers import BusStopHandler
from handlers import TramStopHandler
from handlers import RelationHandler
from handlers import WayHandler
from MultiHandler import MultiHandler
import psycopg2

from optparse import OptionParser

parser = OptionParser()
parser.add_option("-f", "--file", dest="filename", default="map.osm", help="name of input XML file, default: map.osm")
parser.add_option("-d", "--dbname", dest="dbname", default="db_gis_project", help="database name, default: db_gis_project")
parser.add_option("-u", "--user", dest="dbuser", default="kml", help="database username, default: kml")
parser.add_option("-c", "--create", action="store_true", dest="create", default=False, help="creates db structure ")
parser.add_option("-p", "--print", action="store_true", dest="print_data", default=False, help="prints data from xml file ")
parser.add_option("-i", "--import", action="store_true", dest="import_data", default=False, help="imports data from xml file to database ")
parser.add_option("--update_distance", action="store_true", dest="update_distance", default=False, help="updates time of transfer between small_stops ")


(options, args) = parser.parse_args()

findT = TramStopHandler( )
findB = BusStopHandler( )
findR = RelationHandler( )
findW = WayHandler( )

mh = MultiHandler()
mh.handlers.append(findT)
mh.handlers.append(findB)
mh.handlers.append(findR)
mh.handlers.append(findW)

saxparser = make_parser( )
saxparser.setContentHandler(mh)


#tworze sobie slownik (hash table) z wsztstkich wayow z xmla, zeby mozna bylo sprawdzic z jakich nodow sie sklada
ways = dict(findW.ways)

#Wypisuje co mam: nazwa lini, identyfikator w OSM, nody, ktore wchodza w jej sklad i way - dla kazdego waya, nody, ktore wchodza w sklad
def print_xml():
    print "Linie autobusowe"
    for x in findR.busRelations:
        print "Linia: " + x[1]
        print "Identyfikator: " + x[0]
        print "Węzły: "
        print x[2]
        for k in x[3]:
            print "Droga " + k + " z wezlow: "
            if ways.has_key(k):
                print ways[k]
            else:
                print "nie ma drogi w bazie:("
        print
        print
    
    print "Linie tarmwajowe"
    for x in findR.tramRelations:
        print "Linia: " + x[1]
        print "Identyfikator: " + x[0]
        print "Węzły: "
        print x[2]
        for k in x[3]:
            print "Droga " + k + " z wezlow: "
            if ways.has_key(k):
                print ways[k]
            else:
                print "nie ma drogi w bazie:("
        print
        print
    
    #wypisuje przystanki, ktore sparsowano, tupla z (id, nazwa, lon, lat)
    print "Przystanki autobusowe"
    for x in findB.stops:
        print x    
    
    print "Przystanki tramwajowe"
    for x in findT.stops:
        print x  

#dodaje unikalne przystanki
def add_stops(cur):
    for x in findB.stops:
        if not x[1] == '':
            cur.execute("SELECT name FROM stops WHERE name = %s;", (x[1],))
            out = cur.fetchone()
            if out == None:
                cur.execute("INSERT INTO stops (name) VALUES (%s);", (x[1],))
            else:
                print "Stop already exists: " + str(out[0])

#odleglosc w metrach z postgisa
def distance(cur, p1, p2):
    cur.execute("SELECT ST_Distance(\
            ST_Transform(ST_GeomFromText('POINT(%s %s)',4326),26986),\
            ST_Transform(ST_GeomFromText('POINT(%s %s)', 4326),26986)\
            );",(p1[0],p1[1],p2[0],p2[1]))
    out = cur.fetchone()
    return out[0]

#odleglosc miedzy najdalszymi przystankami o tej samej nazwie - predkosc poruszania - 2 m/s
def stop_distance(cur):
    cur.execute("""SELECT a.id, ST_Distance(ST_Transform(a.p2,26986),ST_Transform(b.p1,26986)) FROM (SELECT Stops.id, MAX(ST_AsEWKT(Small_stops.coordinate)) AS p2 FROM Stops JOIN Small_stops ON Stops.id = Small_stops.id_stop GROUP BY Stops.id ORDER BY Stops.id) a  JOIN (SELECT Stops.id, MIN(ST_AsEWKT(Small_stops.coordinate)) AS p1 FROM Stops JOIN Small_stops ON Stops.id = Small_stops.id_stop GROUP BY Stops.id ORDER BY Stops.id) b ON a.id = b.id;""")
    for x in cur.fetchall():
        cur.execute("UPDATE stops SET time=%s WHERE id=%s;",(x[1]/2,x[0]))
        print "Transfer between stops of stop id: " + str(x[0]) + " takes about " + str(x[1]/2) + " seconds"
#small stops
def small_stops(cur):
    for x in findB.stops:
        if not x[1] == '':
            cur.execute("SELECT id FROM small_stops WHERE coordinate = ST_GeomFromText('POINT(%s %s)',4326);",(float(x[2]), float(x[3])))
            out = cur.fetchone()
            if out == None:
                cur.execute("SELECT id FROM stops WHERE name = %s;", (x[1],))
                stopId = cur.fetchone()[0]
                cur.execute("INSERT INTO small_stops (id_stop, add_information, coordinate)\
                            VALUES (%s, %s, ST_GeomFromText('POINT(%s %s)',4326));",(stopId, x[1], float(x[2]), float(x[3])))
            else:
                print "Small stop already exists: id " + str(out[0])

def create_db(cur):
    cur.execute("""CREATE TABLE Stops ( id SERIAL , name varchar(40) UNIQUE NOT NULL, time smallint DEFAULT 0 ,PRIMARY KEY (id) );
                    CREATE TABLE Small_Stops (id SERIAL, id_stop smallint REFERENCES stops (id), add_information varchar (40), PRIMARY KEY (id));
                    SELECT AddGeometryColumn('', 'small_stops', 'coordinate', 4326, 'POINT', 2);
                    CREATE TABLE Lines (id SERIAL, num Smallint NOT NULL, start_stop Smallint REFERENCES small_stops (id), end_stop Integer REFERENCES small_stops (id), PRIMARY KEY (id));
                    CREATE INDEX num_idx ON lines ( num );
                    CREATE TABLE Stop_Lines (id serial,id_line Integer REFERENCES lines (id), stop_id Integer REFERENCES small_stops (id), next Integer REFERENCES small_stops (id),PRIMARY KEY (id));
                    CREATE INDEX id_line_idx ON stop_lines ( id_line );
                    CREATE TABLE Ways (id_way Smallint NOT NULL UNIQUE, PRIMARY KEY (id_way) );
                    SELECT AddGeometryColumn('', 'ways','line',-1,'LINESTRING',2);
                    CREATE TABLE Route_Lines (id SERIAL, id_line Integer REFERENCES lines (id), id_way Integer REFERENCES ways (id_way), next Integer REFERENCES ways (id_way), PRIMARY KEY (id));
                    CREATE INDEX id_line_idx2 ON route_lines ( id_line );""")

def main():
    if options.create:
        conn = psycopg2.connect(database=options.dbname, user=options.dbuser)
        print "connected " + options.dbname
        cur = conn.cursor()
        create_db(cur)
        cur.close()
        conn.close()
        print "connection closed"
        return 0
    if options.print_data:
        saxparser.parse(open(options.filename,'r'))
        print_xml()
        return 0
    if options.import_data:
        saxparser.parse(open(options.filename,'r'))
        conn = psycopg2.connect(database=options.dbname, user=options.dbuser)
        print "connected " + options.dbname
        cur = conn.cursor()
        add_stops(cur)
        conn.commit()
        small_stops(cur)
        conn.commit()
        if options.update_distance:
            stop_distance(cur)
            conn.commit()
        cur.close()
        conn.close()
        print "import successful: connection closed"
        return 0
    if options.update_distance:
        conn = psycopg2.connect(database=options.dbname, user=options.dbuser)
        cur = conn.cursor()
        print "connected " + options.dbname
        stop_distance(cur)
        conn.commit()
        cur.close()
        conn.close()
        print "update successful: connection closed"
        return 0
    
    parser.print_help()
    return 0
        
if __name__ == '__main__':
    main()